unit uEditarUnidades;

  {$MODE Delphi}

interface

uses
  {$IFDEF WINDOWS}
  Windows,
  {$ELSE}
  LCLType,
  {$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, ExtCtrls, uBaseFormularios, uActores, utilidades, ufichasLPD,
  uEditarFichaUnidades, uCosa, uVisorFichasExpandidas, uCosaConNombre,
  uverdoc,
  uSalasDeJuego, uConstantesSimSEE, uOpcionesSimSEEEdit, uAuxiliares, xMatDefs;

resourcestring
  rsCapa = 'Capa';
  rsFechaDeInicio = 'Fecha de Inicio';
  rsNumeroDeMaquinas = 'Número de máquinas';
  rsPeriodicaQ = 'Periodica?';
  mesConfirmaEliminarFicha = '¿Confirma que desea eliminar la ficha seleccionada?';
  mesActorDebeTener1Ficha =
    'El Actor debe tener al menos una ficha indicando la cantidad de unidades disponibles';

type

  { TEditarUnidades }

  TEditarUnidades = class( TBaseFormularios )
    BAyuda: TButton;
    Panel1: TPanel;
    BGuardar: TButton;
    BCancelar: TButton;
    BAgregarFicha: TButton;
    BVerExpandida: TButton;
    sgUnidades: TStringGrid;
    procedure BAyudaClick(Sender: TObject);
    procedure BCancelarClick(Sender: TObject);
    procedure cb_AltaUnidades_CON_INCERTIDUMBREChange(Sender: TObject);
    procedure sgUnidadesDrawCell(Sender: TObject; ACol, ARow: integer;
      Rect: TRect; State: TGridDrawState);
    procedure sgUnidadesMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: integer);
    procedure sgUnidadesMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: integer);
    procedure sgUnidadesMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: integer);
    procedure BAgregarFichaClick(Sender: TObject);
    procedure BGuardarClick(Sender: TObject); override;
    procedure BVerExpandidaClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure sgUnidadesValidarCambio(Sender: TObject);
    procedure sgUnidadesGetEditText(Sender: TObject; ACol, ARow: integer;
      var Value: string);
    procedure sgUnidadesKeyDown(Sender: TObject; var Key: word; Shift: TShiftState);
  private
    lista: TFichasLPD;
    actor: TActor;
    tiposColsUnidades: TDAOfTTipoColumna;
    function editar(fila: integer; clonar: boolean): boolean;
    procedure eliminar(fila: integer);
    procedure actualizarTablaUnidades;
    function validarCambioTablaUnidades(listado: TStringGrid;
      fila, columna: integer): boolean;
    procedure cambioValorTablaUnidades(listado: TStringGrid; fila, columna: integer);
  public
    etiquetasUnidades: TDAofString;
    constructor Create(AOwner: TControl; sala: TSalaDeJuego; actor: TActor;
      listaUnidades: TFichasLPD; etiquetasUnidades: TDAofString); reintroduce;
    function darLista_: TFichasLPD;
  end;

implementation

uses ueditor_resourcestrings, SimSEEEditMain;

{$R *.lfm}

constructor TEditarUnidades.Create(AOwner: TControl; sala: TSalaDeJuego;
  actor: TActor; listaUnidades: TFichasLPD; etiquetasUnidades: TDAofString);
var
  etiquetas: array of string;
begin
  inherited Create(AOwner );
  self.Top := AOwner.Top + plusTop;
  self.Left := AOwner.Left + plusLeft;

  listaUnidades.clearExpanded;
  self.lista := TFichasLPD(listaUnidades.Create_Clone);
  self.actor := actor;
  self.etiquetasUnidades := etiquetasUnidades;

  uCosaConNombre.resolver_referenciasDeCosa(self.lista, sala.listaActores);
  uCosaConNombre.eliminar_referencias_del(self.lista);

  //Aca escribo las etiquetas q salen en el formulario
  setlength(etiquetas, 7);
  etiquetas[0] := encabezadoTextoEditable + rsFechaDeInicio;
  etiquetas[1] := rsNumeroDeMaquinas;
  etiquetas[2] := rsPeriodicaQ;
  etiquetas[3] := rsCapa;
  etiquetas[4] := encabezadoBTEditar;
  etiquetas[5] := encabezadoBTEliminar;
  etiquetas[6] := encabezadoBTClonar;

  utilidades.initListado(sgUnidades, etiquetas, tiposColsUnidades, False);
  actualizarTablaUnidades;
  guardado := True;
end;

function TEditarUnidades.darLista_: TFichasLPD;
begin
  Result := lista;
end;

function TEditarUnidades.editar(fila: integer; clonar: boolean): boolean;
var
  form: TEditarFichaUnidades;
  ficha: TFichaUnidades;
  res: boolean;
begin
  res := False;
  if fila = 0 then
    ficha := nil
  else
    ficha := TFichaUnidades(lista[fila - 1]);

  form := TEditarFichaUnidades.Create(self, actor, ficha, nil, etiquetasUnidades);
  if form.ShowModal = mrOk then
  begin
    if ficha <> nil then
    begin
      if not clonar then
      begin
        lista.Remove(ficha);
        ficha.Free;
      end;
    end;
    lista.add(form.darFicha());
    actualizarTablaUnidades;
    guardado := False;
    res := True;
  end;
  form.Free;
  Result := res;
end;

procedure TEditarUnidades.eliminar(fila: integer);
begin
  if lista.Count > 1 then
  begin
    if (Application.MessageBox(PChar(mesConfirmaEliminarFicha),
      PChar(mesConfirmarEliminacion), MB_OKCANCEL + MB_ICONEXCLAMATION) = idOk) then
    begin
      lista[fila - 1].Free;
      lista.Delete(fila - 1);
      actualizarTablaUnidades;
      guardado := False;
    end;
  end
  else
    ShowMessage(mesActorDebeTener1Ficha);
end;

procedure TEditarUnidades.actualizarTablaUnidades;
var
  i: integer;
  oldItemRow: TCosa;
begin
  if (sgUnidades.row > 0) and (sgUnidades.row < lista.Count) then
    oldItemRow := lista.items[sgUnidades.row - 1]
  else
    oldItemRow := nil;

  lista.SortByFecha;

  sgUnidades.RowCount := lista.Count + 1;
  if sgUnidades.RowCount > 1 then
    sgUnidades.FixedRows := 1
  else
    sgLimpiarSeleccion(sgUnidades);

  for i := 0 to lista.Count - 1 do
  begin
    sgUnidades.cells[0, i + 1] :=
      FSimSEEEdit.fechaIniToString(TFichaUnidades(lista[i]).fecha);
    sgUnidades.cells[1, i + 1] := TFichaUnidades(lista[i]).infoAd_20;
    sgUnidades.Cells[2, i + 1] := boolToSiNo(TFichaLPD(lista[i]).periodicidad <> nil);
    sgUnidades.Cells[3, i + 1] := IntToStr(TFichaUnidades(lista[i]).capa);
  end;

  for i := 0 to sgUnidades.ColCount - 1 do
    utilidades.AutoSizeTypedCol(sgUnidades, i, tiposColsUnidades[i], iconos);

  utilidades.AutoSizeTable(self, sgUnidades, maxAnchoTablaEnorme,
    maxAlturaTablaMediana, TSimSEEEditOptions.getInstance.
    deshabilitarScrollHorizontalEnListados);

  BAgregarFicha.Left := sgUnidades.Left + sgUnidades.Width - BAgregarFicha.Width;
  BVerExpandida.Left := BAgregarFicha.Left - BVerExpandida.Width - 10;

  if oldItemRow <> nil then
    sgUnidades.Row := lista.IndexOf(oldItemRow) + 1;

end;

function TEditarUnidades.validarCambioTablaUnidades(listado: TStringGrid;
  fila, columna: integer): boolean;
begin
  case columna of
    0:
      Result := inherited validarCeldaFecha(listado, fila, columna);
    1:
      Result := inherited validarCeldaNInt(listado, fila, columna);
    else
      Result := True;
  end;
end;

procedure TEditarUnidades.cambioValorTablaUnidades(listado: TStringGrid;
  fila, columna: integer);
begin
  if columna = 0 then //Fecha
  begin
    TFichaUnidades(lista[fila - 1]).fecha.Free;
    TFichaUnidades(lista[fila - 1]).fecha :=
      FSimSEEEdit.StringToFecha(listado.Cells[columna, fila]);
    guardado := False;
    actualizarTablaUnidades;
  end
  else if columna = 1 then //NUnidades
  begin
    TFichaUnidades(lista[fila - 1]).nUnidades[0] :=
      StrToInt(listado.Cells[columna, fila]);
    guardado := False;
  end;
end;

procedure TEditarUnidades.BCancelarClick(Sender: TObject);
begin
  inherited BCancelarClick(Sender);
end;

procedure TEditarUnidades.BAyudaClick(Sender: TObject);
begin
  verdoc(self, TFichaUnidades);
end;

procedure TEditarUnidades.cb_AltaUnidades_CON_INCERTIDUMBREChange(Sender: TObject);
begin
  guardado := False;
end;

procedure TEditarUnidades.sgUnidadesDrawCell(Sender: TObject;
  ACol, ARow: integer; Rect: TRect; State: TGridDrawState);
begin
  utilidades.listadoDrawCell(Sender, ACol, ARow, Rect, State, tiposColsUnidades[ACol],
    nil, iconos, validarCambioTablaUnidades);
end;

procedure TEditarUnidades.sgUnidadesGetEditText(Sender: TObject;
  ACol, ARow: integer; var Value: string);
begin
  utilidades.listadoGetEditText(Sender, ACol, ARow, Value);
end;

procedure TEditarUnidades.sgUnidadesKeyDown(Sender: TObject; var Key: word;
  Shift: TShiftState);
begin
  utilidades.listadoKeyDown(Sender, Key, Shift, tiposColsUnidades,
    self.validarCambioTablaUnidades, self.cambioValorTablaUnidades);
end;

procedure TEditarUnidades.sgUnidadesValidarCambio(Sender: TObject);
begin
  utilidades.listadoValidarCambio(Sender, tiposColsUnidades,
    self.validarCambioTablaUnidades, self.cambioValorTablaUnidades);
end;

procedure TEditarUnidades.sgUnidadesMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  utilidades.ListadoMouseDown(Sender, Button, Shift, X, Y);
end;

procedure TEditarUnidades.sgUnidadesMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: integer);
begin
  utilidades.ListadoMouseMove(Sender, Shift, X, Y, tiposColsUnidades);
end;

procedure TEditarUnidades.sgUnidadesMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: integer);
var
  res: TTipoColumna;
begin
  res := utilidades.ListadoMouseUp(Sender, Button, Shift, X, Y, tiposColsUnidades);
  case res of
    TC_btEditar: editar(TStringGrid(Sender).row, False);
    TC_btEliminar: eliminar(TStringGrid(Sender).row);
    TC_btClonar: if editar(TStringGrid(Sender).row, True) then
        sgUnidades.Row := sgUnidades.RowCount - 1;
  end;
end;

procedure TEditarUnidades.BAgregarFichaClick(Sender: TObject);
begin
  editar(0, False);
end;

procedure TEditarUnidades.BGuardarClick(Sender: TObject);
begin
  ModalResult := mrOk;
end;

procedure TEditarUnidades.BVerExpandidaClick(Sender: TObject);
var
  form: TVisorFichasExpandidas;
begin
  form := TVisorFichasExpandidas.Create(self, actor, lista);
  form.ShowModal;
end;

procedure TEditarUnidades.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  inherited FormCloseQuery(Sender, CanClose);
end;

end.
