unit uTSumComConAlmacenYAgenda;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  uCosa, SysUtils, StdCtrls, uGlobs, uActores, uFechas, unodos, uCosaConNombre,
  uFichasLPD, usimplex, xMatDefs, uauxiliares, unodocombustible,
  uFuentesAleatorias, uTSumComb;


resourcestring
  rsSuministroSimpleCombustible = 'Suministro Combustible Con Almacen y Agenda';


type

  TFichaSumComConAlmacenYAgenda = class(TFichaLPD)
    QMax: NReal;   // Max Unidades / s
    precio: NREal; // USD / UNIDAD
    fd: NReal;
    TMR: NReal;



    VGNL_max: NReal;      //volumen maximo del tanque
    VGNL_min: NReal;      //volumen minimo de GNL que debe quedar en el tanque (toma u otra restriccion)


    constructor Create(capa: integer; fecha: TFecha;
      periodicidad: TPeriodicidad; QMax,
      Precio: NReal;
      indicePreciosPorCombustible: TFuenteAleatoria;
      bornePreciosPorCombustible: string;
      fd, TMR: NReal;
       VGNL_max: NReal;
      VGNL_min: NReal);

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    function InfoAd_: string; override;
  end;

  TSumComConAlmacenYAgenda = class(TSuministroSimpleCombustible)
  public



    // variables
    pa: TFichaSumComConAlmacenYAgenda;

    //volumenes de GNL al inicio y al final del paso
    X_VGNL, Xs_VGNL: NReal; //Xs es final
    VGNL_inicial: NReal;
    Ndiscret: integer;

    fuenteDeAporte: TFuenteAleatoria; //descargas de GNL en el tanque
    nombreBorne: string;
    numeroBorne: integer;

    constructor Create(
       capa: integer; nombre: string; nacimiento, muerte: TFecha;
       lpdUnidades: TFichasLPD;
       lpd: TFichasLPD;
       NodoSumComb: TNodoCombustible;
       fuenteDeAporte: TFuenteAleatoria;
      nombreBorne: string;
//      numeroBorne: integer;

       );

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure PubliVars; override;
    procedure PrepararMemoria(globs: TGlobs); override;

    class function DescClase: string; override;

    procedure RegistrarParametrosDinamicos; override;

    procedure prepararPaso_as; virtual; // preparación previa a los sorteos.
    procedure SorteosDelPaso(sortear: boolean); virtual;
    procedure PrepararPaso_ps; virtual;

    procedure opt_nvers(var ivar, ivae, ires: integer); virtual;
    procedure opt_cargue(s: TSimplex); virtual;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); virtual;
    procedure opt_leerSolucion(s: TSimplex); virtual;


    procedure Free; override;




  end;

procedure AlInicio;
procedure ALFinal;

implementation



// Métodos de TSuministroCombustible

//function TSuministroCombustible.referenciaAlNodo(NodoSumComb:
//  TNodoCombustible): boolean;
//begin
//  Result := self.NodoCombustible =  NodoSumComb;
//end;
//
//constructor TSuministroCombustible.Create_ReadFromText(f: TArchiTexto);
//begin
//  inherited Create_ReadFromText( f );
//  f.rdReferencia( 'NodoCombustible', TCosa( NodoCombustible ), self );
//end;
//
//procedure TSuministroCombustible.WriteToText(f: TArchiTexto);
//begin
//  inherited WriteToText( f );
//  f.wrReferencia( 'NodoCombustible', Self.NodoCombustible );
//end;




// Métodos de TFichaSumComConAlmacenYAgenda

constructor TFichaSumComConAlmacenYAgenda.Create(capa: integer;
  fecha: TFecha; periodicidad: TPeriodicidad; QMax,
  Precio: NReal;
  indicePreciosPorCombustible: TFuenteAleatoria;
  bornePreciosPorCombustible: string;
  fd, TMR: NReal;
  fuenteDeAporte: TFuenteAleatoria;
  nombreBorne: string;
//  numeroBorne: integer;
  VGNL_max: NReal;
  VGNL_min: NReal);

begin
  inherited Create(capa, fecha, periodicidad);
  self.Precio := Precio;
  self.indicePreciosPorCombustible := indicePreciosPorCombustible;
  self.bornePreciosPorCombustible :=  bornePreciosPorCombustible;
  self.fd := fd;
  self.TMR := TMR;
  self.fuenteDeAporte := fuenteDeAporte;
  self.nombreBorne := nombreBorne;
//  self.numeroBorne := numeroBorne;
  self.VGNL_max := VGNL_max;
  self.VGNL_min := VGNL_min;
end;

constructor TFichaSumComConAlmacenYAgenda.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText( f );
  f.IniciarLecturaRetrasada;
  f.rd('QMax', QMax);
  f.rd('Precio', Precio);
  f.rd( 'fd', fd );
  f.rd( 'TMR', TMR );
  f.rdReferencia('indicePreciosPorCombustible',
    TCosa(indicePreciosPorCombustible), self);
  f.rd('bornePreciosPorCombustible', bornePreciosPorCombustible);
  f.rd('fuenteDeAporte', TCosa(fuenteDeAporte));
  f.rd( 'nombreBorne', nombreBorne );
//  f.rd( 'numeroBorne', numeroBorne );
  f.rd( 'VGNL_max',VGNL_max);
  f.rd( 'VGNL_min',VGNL_min);
  f.EjecutarLectura;
end;

procedure TFichaSumComConAlmacenYAgenda.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText( f );
  f.wr('QMax', QMax);
  f.wr('Precio', Precio);
  f.wr( 'fd', fd );
  f.wr( 'TMR', TMR );
  f.wrReferencia('indicePreciosPorCombustible', indicePreciosPorCombustible);
  f.wr('bornePreciosPorCombustible', bornePreciosPorCombustible);
  f.wr('fuenteDeAporte', fuenteDeAporte);
  f.wr('nombreBorne', nombreBorne);
//  f.wr( 'numeroBorne', numeroBorne );
  f.wr( 'VGNL_max',VGNL_max);
  f.wr( 'VGNL_min',VGNL_min);

end;

function TFichaSumComConAlmacenYAgenda.InfoAd_: string;
begin
  Result := 'QMax: ' + FloatToStr(QMax) + ', Precio:' + FloatToStr(Precio);
end;




//--------------------------
// Métodos de TSumComConAlmacenYAgenda
//==========================

constructor TSumComConAlmacenYAgenda.Create(capa: integer;
  nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades: TFichasLPD;
  lpd: TFichasLPD;
  NodoSumComb: TNodoCombustible);

begin

  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, lpd, NodoSumComb);
end;

constructor TSumComConAlmacenYAgenda.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.EjecutarLectura;
end;

procedure TSumComConAlmacenYAgenda.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);

end;

procedure TSumComConAlmacenYAgenda.PubliVars;
begin
  inherited PubliVars;
end;





class function TSumComConAlmacenYAgenda.DescClase: string;
begin
  result:= rsSuministroSimpleCombustible;
end;

procedure TSumComConAlmacenYAgenda.RegistrarParametrosDinamicos;
begin
  inherited registrarParametrosDinamicos;
  lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pA, nil);
end;




procedure TSumComConAlmacenYAgenda.sim_iniciocronica;
begin
  X_VGNL := self.VGNL_inicial;

end;

procedure TSumComConAlmacenYAgenda.opt_nvers(var ivar, ivae, ires: integer);
begin
  self.ivar := ivar;
  Inc(ivar, globs.npostes+1);

  self.ires:=ires;
  Inc(ires, 1); //las restricciones son las ecuaciones de  y Xs_VGNL = f(X,u,r)
end;

procedure TSumComConAlmacenYAgenda.opt_cargue(s: TSimplex);
var
  iresnodo: integer;
  kposte: integer;
  VAporte: NReal;
begin
  iresnodo := self.NodoCombustible.ires;
  for kposte := 0 to globs.npostes - 1 do
  begin
    s.pon_e(iresnodo + kposte , ivar + kposte, 1);

    s.pon_e(ires, ivar + kposte, -globs.DurPos[kposte]);// Xs_VGNL - Vgnl_min>0 (0 = X_VGNL + Aportes - Q*durpos - Qbog*durpos - Xs_VGNL)
    VAporte := pa.fuenteDeAporte.bornera[pa.numeroBorne];
    s.pon_e(ires, s.nc, self.Xs_VGNL +  VAporte - pa.Vgnl_min);        //Qbog=0


 // cargar funcion objetivo -costo
    s.pon_e(s.nf, ivar + kposte, -PrecioOperativo * globs.durpos[kposte]);
  end;


end;

procedure TSumComConAlmacenYAgenda.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  kposte: integer;
begin
  for kposte := 0 to globs.npostes - 1 do
    s.cota_sup_set(ivar + kposte, QMaxOperativo);
end;

procedure TSumComConAlmacenYAgenda.opt_leerSolucion(s: TSimplex);
var
  kposte: integer;
  Consumo: NREal;
begin
  Consumo:= 0;
  for kposte := 0 to globs.npostes - 1 do
  begin
    Q[kposte] := s.xval(ivar + kposte);
    Consumo:= Consumo + Q[kposte] * globs.DurPos[kposte];



  end;

  X_VGNL := s.yval(ires) + pa.Vgnl_min;

  CostoDirectoDelPaso :=  Consumo * PrecioOperativo;
end;


procedure TSumComConAlmacenYAgenda.Free;
begin
  setlength(Q, 0);
  inherited Free;
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TSumComConAlmacenYAgenda.ClassName,
    TSumComConAlmacenYAgenda);
  registrarClaseDeCosa(TFichaSumComConAlmacenYAgenda.ClassName,
    TFichaSumComConAlmacenYAgenda);

end;

procedure AlFinal;
begin
end;

end.
